@@ -53,46 +53,53 @@ from six.moves import range
 DEFAULT_QUEUE = 'General'
 """ Default queue used. """
 
-class AuthorizationError(Exception):
+ALL_QUEUES = object()
+
+class RtError(Exception):
+    """ Super class of all Rt Errors """
+
+    pass
+
+class AuthorizationError(RtError):
     """ Exception raised when module cannot access :term:`API` due to invalid
     or missing credentials. """
 
     pass
 
-class NotAllowed(Exception):
+class NotAllowed(RtError):
     """ Exception raised when request cannot be finished due to
     insufficient privileges. """
 
     pass
 
-class UnexpectedResponse(Exception):
+class UnexpectedResponse(RtError):
     """ Exception raised when unexpected HTTP code is received. """
 
     pass
 
-class UnexpectedMessageFormat(Exception):
+class UnexpectedMessageFormat(RtError):
     """ Exception raised when response has bad status code (not the HTTP code,
     but code in the first line of the body as 200 in `RT/4.0.7 200 Ok`)
     or message parsing fails because of unexpected format. """
 
     pass
 
-class APISyntaxError(Exception):
+class APISyntaxError(RtError):
     """ Exception raised when syntax error is received. """
 
     pass
 
-class InvalidUse(Exception):
+class InvalidUse(RtError):
     """ Exception raised when API method is not used correctly. """
 
     pass
 
-class BadRequest(Exception):
+class BadRequest(RtError):
     """ Exception raised when HTTP code 400 (Bad Request) is received. """
 
     pass
 
-class ConnectionError(Exception):
+class ConnectionError(RtError):
     """ Encapsulation of various exceptions indicating network problems. """
 
     def __init__(self, message, cause):
@@ -104,7 +111,7 @@ class ConnectionError(Exception):
         super(ConnectionError, self).__init__(message + ' (Caused by ' + repr(cause) + ")")
         self.cause = cause
 
-class InvalidQueryError(Exception):
+class InvalidQueryError(RtError):
     """ Exception raised when attempting to search RT with an invalid raw query. """
 
     pass
@@ -129,7 +136,7 @@ class Rt:
         'content_pattern': re.compile('Content:'),
         'content_pattern_bytes': re.compile(b'Content:'),
         'attachments_pattern': re.compile('Attachments:'),
-        'attachments_list_pattern': re.compile(r'[^0-9]*(\d+): ([\w().]+) \((\w+/[\w.]+) / ([0-9a-z.]+)\),?$'),
+        'attachments_list_pattern': re.compile(r'[^0-9]*(\d+): (.+) \((.+) / (.+)\),?$'),
         'headers_pattern_bytes': re.compile(b'Headers:'),
         'links_updated_pattern': re.compile('^# Links for ticket [0-9]+ updated.$'),
         'created_link_pattern': re.compile('.* Created link '),
@@ -185,7 +192,7 @@ class Rt:
             # assured, that we are login in instantly)
             self.login_result = True
 
-    def __request(self, selector, post_data={}, files=[], without_login=False,
+    def __request(self, selector, get_params={}, post_data={}, files=[], without_login=False,
                   text_response=True):
         """ General request for :term:`API`.
  
@@ -215,7 +222,7 @@ class Rt:
                 if post_data:
                     response = self.session.post(url, data=post_data)
                 else:
-                    response = self.session.get(url)
+                    response = self.session.get(url, params=get_params)
             else:
                 files_data = {}
                 for i, file_pair in enumerate(files):
@@ -238,7 +245,7 @@ class Rt:
                     raise UnexpectedResponse('Unknown response encoding (UTF-8 does not work).')
                 else:
                     # replace errors - we need decoded content just to check for error codes in __check_response
-                    result = response.content.decode('utf-8', errors='replace')
+                    result = response.content.decode('utf-8', 'replace')
             self.__check_response(result)
             if not text_response:
                 return response.content
@@ -376,7 +383,9 @@ class Rt:
             >>> tickets = tracker.search(CF_Domain='example.com', Subject__like='warning')
             >>> tickets = tracker.search(Queue='General', order='Status', raw_query="id='1'+OR+id='2'+OR+id='3'")
 
-        :keyword Queue:      Queue where to search
+        :keyword Queue:      Queue where to search. If you wish to search across
+                             all of your queues, pass the ALL_QUEUES object as the
+                             argument.
         :keyword order:      Name of field sorting result list, for descending
                              order put - before the field name. E.g. -Created
                              will put the newest tickets at the beginning
@@ -413,7 +422,11 @@ class Rt:
         :raises:  UnexpectedMessageFormat: Unexpected format of returned message.
                   InvalidQueryError: If raw query is malformed
         """
-        query = 'search/ticket?query=(Queue=\'%s\')' % (Queue or self.default_queue,)
+        get_params = {}
+        query = []
+        url = 'search/ticket'
+        if Queue is not ALL_QUEUES:
+            query.append("Queue=\'%s\'" % (Queue or self.default_queue))
         if not raw_query:
             operators_map = {
                 'gt':'>',
@@ -431,16 +444,17 @@ class Rt:
                     key = '__'.join(key_parts[:-1])
                     op = operators_map.get(key_parts[-1], '=')
                 if key[:3] != 'CF_':
-                    query += "+AND+(%s%s\'%s\')" % (key, op, value)
+                    query.append("%s%s\'%s\'" % (key, op, value))
                 else:
-                    query += "+AND+(CF.{%s}%s\'%s\')" % (key[3:], op, value)
+                    query.append("CF.{%s}%s\'%s\'" % (key[3:], op, value))
         else:
-            query += '+AND+(' + raw_query + ')'
+            query.append(raw_query)
+        get_params['query'] = ' AND '.join('(' + part + ')' for part in query)
         if order:
-            query += "&orderby=" + order
-        query += "&format=l"
+            get_params['orderby'] = order
+        get_params['format'] = 'l'
 
-        msg = self.__request(query)
+        msg = self.__request(url, get_params=get_params)
         lines = msg.split('\n')
         if len(lines) > 2:
             if self.__get_status_code(lines[0]) != 200 and lines[2].startswith('Invalid query: '):
@@ -580,7 +594,7 @@ class Rt:
                 post_data += "%s: %s\n"%(key, kwargs[key])
             else:
                 post_data += "CF.{%s}: %s\n"%(key[3:], kwargs[key])
-        msg = self.__request('ticket/new', {'content':post_data})
+        msg = self.__request('ticket/new', post_data={'content':post_data})
         state = msg.split('\n')[2]
         res = re.search(' [0-9]+ ',state)
         if res is not None:
@@ -615,7 +629,7 @@ class Rt:
                 post_data += "%s: %s\n"%(key, value)
             else:
                 post_data += "CF.{%s}: %s\n" % (key[3:], value)
-        msg = self.__request('ticket/%s/edit' % (str(ticket_id)), {'content':post_data})
+        msg = self.__request('ticket/%s/edit' % (str(ticket_id)), post_data={'content':post_data})
         state = msg.split('\n')[2]
         return self.RE_PATTERNS['update_pattern'].match(state) is not None
 
@@ -729,8 +743,8 @@ class Rt:
         :keyword text: Content of email message
         :keyword cc: Carbon copy just for this reply
         :keyword bcc: Blind carbon copy just for this reply
-        :keyword files: List of pairs (filename, file-like object) describing
-                        files to attach as multipart/form-data
+        :keyword files: Files to attach as multipart/form-data
+                        List of 2/3 tuples: (filename, file-like object, [content type])
         :returns: ``True``
                       Operation was successful
                   ``False``
@@ -742,10 +756,10 @@ Action: correspond
 Text: %s
 Cc: %s
 Bcc: %s"""%(str(ticket_id), re.sub(r'\n', r'\n      ', text), cc, bcc)}
-        for file_pair in files:
-            post_data['content'] += "\nAttachment: %s" % (file_pair[0],)
+        for file_info in files:
+            post_data['content'] += "\nAttachment: %s" % (file_info[0],)
         msg = self.__request('ticket/%s/comment' % (str(ticket_id),),
-                             post_data, files)
+                             post_data=post_data, files=files)
         return self.__get_status_code(msg) == 200
 
     def comment(self, ticket_id, text='', cc='', bcc='', files=[]):
@@ -772,8 +786,8 @@ Bcc: %s"""%(str(ticket_id), re.sub(r'\n', r'\n      ', text), cc, bcc)}
 
         :param ticket_id: ID of ticket to which comment belongs
         :keyword text: Content of comment
-        :keyword files: List of pairs (filename, file-like object) describing
-                        files to attach as multipart/form-data
+        :keyword files: Files to attach as multipart/form-data
+                        List of 2/3 tuples: (filename, file-like object, [content type])
         :returns: ``True``
                       Operation was successful
                   ``False``
@@ -783,10 +797,10 @@ Bcc: %s"""%(str(ticket_id), re.sub(r'\n', r'\n      ', text), cc, bcc)}
         post_data = {'content':"""id: %s
 Action: comment
 Text: %s""" % (str(ticket_id), re.sub(r'\n', r'\n      ', text))}
-        for file_pair in files:
-            post_data['content'] += "\nAttachment: %s" % (file_pair[0],)
+        for file_info in files:
+            post_data['content'] += "\nAttachment: %s" % (file_info[0],)
         msg = self.__request('ticket/%s/comment' % (str(ticket_id),),
-                             post_data, files)
+                             post_data=post_data, files=files)
         return self.__get_status_code(msg) == 200
 
 
@@ -1058,7 +1072,7 @@ Text: %s""" % (str(ticket_id), re.sub(r'\n', r'\n      ', text))}
         post_data = 'id: user/%s\n' % str(user_id)
         for key, val in iteritems(kwargs):
             post_data += '%s: %s\n' % (key, val)
-        msg = self.__request('edit', {'content': post_data})
+        msg = self.__request('edit', post_data={'content': post_data})
         msgs = msg.split('\n')
         if (self.__get_status_code(msg) == 200) and (len(msgs) > 2):
             match = self.RE_PATTERNS['user_pattern'].match(msgs[2])
@@ -1130,7 +1144,7 @@ Text: %s""" % (str(ticket_id), re.sub(r'\n', r'\n      ', text))}
         post_data = 'id: queue/%s\n' % str(queue_id)
         for key, val in iteritems(kwargs):
             post_data += '%s: %s\n' % (key, val)
-        msg = self.__request('edit', {'content': post_data})
+        msg = self.__request('edit', post_data={'content': post_data})
         msgs = msg.split('\n')
         if (self.__get_status_code(msg) == 200) and (len(msgs) > 2):
             match = self.RE_PATTERNS['queue_pattern'].match(msgs[2])
@@ -1221,7 +1235,7 @@ Text: %s""" % (str(ticket_id), re.sub(r'\n', r'\n      ', text))}
         for key in kwargs:
             post_data += "%s: %s\n"%(key, str(kwargs[key]))
         msg = self.__request('ticket/%s/links' % (str(ticket_id),),
-                             {'content':post_data})
+                             post_data={'content':post_data})
         state = msg.split('\n')[2]
         return self.RE_PATTERNS['links_updated_pattern'].match(state) is not None
 
@@ -1250,7 +1264,7 @@ Text: %s""" % (str(ticket_id), re.sub(r'\n', r'\n      ', text))}
                      'id': ticket_id,
                      'del': 1 if delete else 0
                     }
-        msg = self.__request('ticket/link', post_data)
+        msg = self.__request('ticket/link', post_data=post_data)
         state = msg.split('\n')[2]
         if delete:
             return self.RE_PATTERNS['deleted_link_pattern'].match(state) is not None
